@@ -17,13 +17,13 @@ NULL
 #' @rdname angle-conversion
 #' @export
 rad2deg <- function(rad) {
-  stopifnot(is.numeric(rad))
+  # stopifnot(is.numeric(rad))
   rad * 180 / pi
 }
 #' @rdname angle-conversion
 #' @export
 deg2rad <- function(deg) {
-  stopifnot(is.numeric(deg))
+  # stopifnot(is.numeric(deg))
   deg * pi / 180
 }
 
@@ -40,6 +40,15 @@ deg2rad <- function(deg) {
 #' @name trigon
 NULL
 
+dir2ax <- function(x) {
+  (x / 2) %% 180
+}
+
+ax2dir <- function(x) {
+  (2 * x) %% 360
+}
+
+
 #' @rdname trigon
 sind <- function(x) {
   sinpi(x / 180)
@@ -100,17 +109,24 @@ NULL
 #' @rdname spec_atan
 #' @export
 atan2_spec <- function(x, y) {
-  if (y > 0 && x >= 0) {
-    atan(x / y)
-  } else if (y == 0 && x > 0) {
-    pi / 2
-  } else if (y < 0) {
-    atan(x / y + pi)
-  } else if (y > 0 && x < 0) {
-    atan(x / y) + 2 * pi
-  } else if (y == 0 && x == 0) {
-    Inf
-  }
+  # if (y > 0 && x >= 0) {
+  #   atan(x / y)
+  # } else if (y == 0 && x > 0) {
+  #   pi / 2
+  # } else if (y < 0) {
+  #   atan(x / y + pi)
+  # } else if (y > 0 && x < 0) {
+  #   atan(x / y) + 2 * pi
+  # } else if (y == 0 && x == 0) {
+  #   Inf
+  # }
+  dplyr::case_when(
+    y > 0 && x >= 0 ~ atan(x / y),
+    y == 0 && x > 0 ~ pi / 2,
+    y < 0 ~ atan(x / y + pi),
+    y > 0 && x < 0 ~ atan(x / y) + 2 * pi,
+    y == 0 && x == 0 ~ Inf
+  )
 }
 
 #' @rdname spec_atan
@@ -284,10 +300,10 @@ dist_greatcircle <- function(lat1, lon1, lat2, lon2,
   method <- match.arg(method)
   stopifnot(is.numeric(r), length(lat1) == length(lon1), length(lat2) == length(lat2))
 
-  lat1 <- lat1 * pi / 180
-  lon1 <- lon1 * pi / 180
-  lat2 <- lat2 * pi / 180
-  lon2 <- lon2 * pi / 180
+  lat1 <- deg2rad(lat1)
+  lon1 <- deg2rad(lon1)
+  lat2 <- deg2rad(lat2)
+  lon2 <- deg2rad(lon2)
 
   if (method == "haversine") {
     d <- haversine(lat1, lon1, lat2, lon2) * r
@@ -330,16 +346,16 @@ dist_greatcircle <- function(lat1, lon1, lat2, lon2,
 #' tokyo <- c(35.7, 139.767) # Tokyo
 #' get_azimuth(berlin[1], berlin[2], tokyo[1], tokyo[2])
 get_azimuth <- function(lat_a, lon_a, lat_b, lon_b) {
-  la <- pi / 180 * lat_a
-  lb <- pi / 180 * lat_b
+  la <- deg2rad(lat_a)
+  lb <- deg2rad(lat_b)
 
-  dphi <- (lon_b - lon_a) * (pi / 180)
+  dphi <- deg2rad(lon_b - lon_a)
   cos_lb <- cos(lb)
 
   y <- sin(dphi) * cos_lb
   x <- cos(la) * sin(lb) - sin(la) * cos_lb * cos(dphi)
-  # theta <- atan2d(y, x)
-  theta <- atand(y / x) + 360
+  theta <- atan2d(y, x)
+  # theta <- atand(y / x) + 360
 
   (theta + 360) %% 360
 }
